# --- [ cl_edit ] --------------------------------------------------------------
#
# Textausgabefelder werden mit vertikalem und optional
# vertikalem Scrollbar zusammengestellt
# Ein Button zum Löschen des gesamten Fensterinhaltes ist eingebaut.

import tkinter as Tk

class Editor(Tk.Text):
    
    def __init__(self, master,
                 row = 0,
                 column = 0,
                 rowspan = 1,
                 width  = 40,
                 height = 10,
                 xscroll = True,
                 **kw):
        
        Tk.Text.__init__(self, master,
                         height = height,
                         width = width,
                         **kw)
        
        self.grid(column = column, row = row, rowspan = rowspan, padx = 5, pady = 5, sticky = "nsew")

        # Der Editor erhält einen Scrollbar für die vertikale Richtung.
        yscrollbar=Tk.Scrollbar(master)
        yscrollbar.grid(column = column + 1, row = row, rowspan = rowspan, pady = 5, sticky = "nsw")
        yscrollbar.config(command = self.yview)
        self.configure(yscrollcommand = yscrollbar.set)

        # Und ggf. auch einen Scrollbar für die horizontale Richtung.
        # und einen Button zum Löschen der Textausgabe
        if xscroll == True:
            xscrollbar=Tk.Scrollbar(master, orient = "horizontal", width = 20)
            xscrollbar.grid(column = column, row = row + rowspan, padx = 5, sticky = "new")
            xscrollbar.config(command = self.xview)     

            self.configure(xscrollcommand = xscrollbar.set, wrap = "none")

            self.__btn = Tk.Button(master, text = "X", height = 1, command = self.__delete)
            self.__btn.grid(column = column + 1, row = row + rowspan, sticky = "nw")

        else:
            self.__btn = Tk.Button(master, text = "Delete Log", command = self.__delete)
            self.__btn.grid(column = column, row = row + rowspan, columnspan = 2, sticky = "nsew")
            
    def __delete(self):
        self.delete("1.0", 'end')
        

if __name__ == "__main__":

    top = Tk.Tk()
    top.title("Editor - Test")

    frame1 = Tk.LabelFrame(top, padx = 5, pady = 5, text = "Test", height = 100, relief = "groove")
    frame1.grid(column = 0, row = 0, padx = 5, pady = 5)

    tx1 = Editor(frame1, column = 0, row = 0, rowspan = 4, width = 20, height = 40)
    tx2 = Editor(frame1, column = 2, row = 0, width = 40, height = 10, xscroll = False)

    lbl1 = Tk.LabelFrame(frame1, text = "Frame 1", height = 300)
    lbl1.grid(column = 2, row = 2, columnspan = 2, padx = 5, pady = 5, sticky = "nsew")
    lbl1.grid_propagate(0)

    txt1 = Tk.Label(lbl1, text = "Label 1 in row 2\nund Column 2 und Column 3\nvon Frame1", anchor = "w")
    txt1.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "n")

    lbl2 = Tk.LabelFrame(frame1, text = "Frame 2", height = 100)
    lbl2.grid(column = 2, row = 3, padx = 5, pady = 5, sticky = "nsew")
    lbl2.grid_propagate(0)

    txt3 = Tk.Label(lbl2, text = "Label 2 in row 2\nund Column 3\nvon Frame1", anchor = "w")
    txt3.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "n")
    
    lbl3 = Tk.Frame(frame1)
    lbl3.grid(column = 2, row = 4, sticky = "nsew")

    txt2 = Tk.Label(lbl3, text = "Label 3 in Column 2, Row 4", anchor = 'e', width = 30, bg = "yellow")
    txt2.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "e")
    top.mainloop()
               
