# --- [ cmd_line.py ] -------------------------------------------------------------
#
# Ist zuständig für die Interpretation eines Eingabestrings

def split_string(str):
    """ Zerlegt einen durch Komma getrennten Eingabestring in eine Liste.
        Prüft alle Elemente daraufhin, ob sie eine gültige Zahl enthalten.
        Ggf. mit nachfolgendem "|x" für eine geforderte Bytezahl,

        - 0xAA|4 -> 0x00, 0x00, 0x00, 0xAA

        oder mit führendem
        
        - 0x - hexadezimal
        - 0b - binär
        - 0o - Oktal
        
        Sobald ein Element nicht in eine Zahl umgewandelt werden kann, wird
        dieses Element und der gesamte Rest als Zeichenfolge übernommen.
        
        Rückgabe:
        ein Tuple (Anzahl der Elemente, [Bytearray])
        wenn die Anzahl der Elemente == 0, dann liegt eine leere Eingabe vor """
    
    str = str.strip(',')                # Kommata am Anfang und Ende entfernen

    if str:                             # wenn dar String nicht leer ist
        if str[0] == '\\':              # ein Backslash ist
            list = bytearray()
            for i in range(1, len(str)):
                list.append(ord(str[i]))    # alle Chr anhängen
            return list

    s = str.split(',')                  # zerlegt den String in die Liste s[]
    l = 0                               # Zähler
    list = bytearray()                  # sammelt das Ergebnis als Bytearray
    search_num = True                   # Flag
    


    for i in s:
        # Sobald ein Element nicht in eine Zahl umgewandelt werden konnte
        # wird der gesamte Rest als Zeichenfolge übernommen
            
        if search_num == False:         # es wird nicht weiter auf Ziffern analysiert
            list.append(ord(','))       # das extrahierte Komma wieder einfügen
            for a in i:  
                list.append(ord(a))

            continue
            
        # eine leere Eingabe, ab sofort wird nur noch Text übernommen
        if len(i) == 0:
            search_num = False
            continue
            
        # mit einem Sonderzeichen kann die Anzahl der Bytes einer
        # Zahl festgelegt werden : "34|4" -> 0,0,0,34
            
        sign = i.find('|')
 
        if sign > -1 and sign == len(i) - 2:

            e = int((i[-1:]))
            if not e in (1, 2, 4):              # das Zeichen nach dem '|' prüfen
                for a in i:                     # wenn unzulässig, dann als Text übernehmen
                    list.append(ord(a))
                l += 1
                search_num = False              # ab sofort nur noch Text einlesen
                continue
                     
            ii = (i[:sign])                     # den Teil vor dem '|' herauslösen
                

            try:
                if ii.isdigit():                # und prüfen
                    z = int(ii)
                elif ii[:2] == "0b":
                    z = int(ii, 2)
                elif ii[:2] == "0x":
                    z = int(ii, 0)
                elif ii[:2] == "0o":
                    z = int(ii, 8)
                else:                           # dann wird ein String übernommen
                    for a in i:
                        list.append(ord(a))
                    l += 1
                    search_num = False          # ab sofort nur noch Text einlesen
                    continue
                                                 
                while e > 1:
                    teiler = 256 ** (e -1)
                    list.append( z // teiler)
                    z = z % teiler
                    e -= 1
                        
                list.append(z % 256)

            except:
                for a in i:
                    list.append(ord(a))
                l += 1
                search_num = False              # ab sofort nur noch Text einlesen
                continue               
               
        else:                                   # kein '|' gefunden
            try:
                if i.isnumeric():
                    z = int(i)
                elif i[:2] == "0b":
                    z = int(i, 2)
                elif i[:2] == "0x":
                    z = int(i, 0)
                elif i[:2] == "0o":
                    z = int(i, 8)
                else:
                    for a in i:
                        list.append(ord(a))
                    l += 1
                    search_num = False          # ab sofort nur noch Text einlesen
                    continue

                if z >= 256 ** 2:               # 4 Byte erforderlich
                    e = 3
                    while e > 0:
                        teiler = 256 ** e
                        list.append( z // teiler)
                        z = z % teiler
                        e -= 1
                    
                    list.append( z %  256)

                elif z > 255:                   # 2 Byte erforderlich
                    list.append(z // 256)
                    list.append(z % 256)

                else:                           # 1 Byte reicht aus
                    list.append(z)
  

            except:
                for a in i:
                    list.append(ord(a))
                l += 1
                search_num = False              # ab sofort nur noch Text einlesen
                continue

        l += 1
             
         
    return (list)


if __name__ == "__main__":

    while True:
        a = input("Eingabe : ")
         
        a = (split_string(a))
        print(a)
        z = 0
        for i in a:
            z += i
            print(chr(i), end = "")
        print()

        
# --- [ eof ] ------------------------------------------------------------------
