# --- [ config.py ] ------------------------------------------------------------
#

import tkinter as Tk
from tkinter import ttk

import inputs
import frames
import receive


class RX_Register(object):
    
    
    """ In ein bestehendes Widget werden beliebig viele Checkboxen zur Auswahl
        von Einstellungen plaziert.
        Die Einstellungen werden in Klassenvariablen gespeichert.
        Die Checkbox erhalten aufsteigende "onvalues".
        Die gesamte Einstellung wird in einer Int-Variablen übergeben,
        in der jede Checkbox durch ein Bit repräsentiert wird.
    """

    # Tuple aus Tuplen("Bezeichnung des Buttons", Vorgabewert, Column, Row)
    items = (("0x88 Local AT Response",       1, 0, 0),
             ("0x8A Modem Status",            1, 0, 2),
             ("0x8B Transmit Status",         1, 1, 0),
             ("0x90 Receive Packet",          1, 1, 1),
             ("0x92 IO-Data Sample",          1, 1, 2),
             ("0x95 Node Identification",     1, 2, 0),
             ("0x97 Remote AT Response",      1, 0, 1),
             ("0xA1 Route Record Indicator",  1, 2, 1),
             ("0xA3 Route Request Indicator", 1, 2, 2),
             ("Get Source's Signal Strength", 0, 0, 3))

    # für jede Checkbox eine Variable für den aktuellen Wert anlegen 
    chk_var = [None] * len(items)

    # damit von aussen
    my_self = None
    
    # die Variablen mit den Vorgabewerten aus den "items" initialisieren
    for i in range(len(items)):
        if items[i][1] > 0:
            chk_var[i] = 2 ** i 
        else:
            chk_var[i] = 0

    def put_Values():
        """ Liest die in den Klassenvariablen gespeicherten Einstellungen aus
            und schreibt sie in das Modul, das die Einstellungen benötigt
        """
        s = 0
        for i in range(len(__class__.items)):
            s += __class__.chk_var[i]
    
        frames.show_frame = s
        receive.show_frame = s

    def __init__(self, master, top_level = None, width = 29):
        
        self.frame =ttk.LabelFrame(master, text = "Show RX-Frames")
        self.frame.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")
        
        self.top_level = top_level
        __class__.my_self = self
        self.len_items = len(__class__.items)
        self.chk_var = [None] * self.len_items
        
        for i in range(self.len_items):
            self.chk_var[i] = Tk.IntVar()
            self.chk_var[i].set(self.__class__.chk_var[i])
            
            ttk.Checkbutton(self.frame, text = __class__.items[i][0],
                            variable = self.chk_var[i],
                            width = width,
                            onvalue = 2** i,
                            command = self.dummy).grid(
                            column = __class__.items[i][2],
                            row = __class__.items[i][3],
                            padx = 5,
                            pady = 5,
                            sticky = "nsw")

        ttk.Button(self.frame, text = "übernehmen", command = self.ende).grid(
                    column = 1, row = 3, padx = 5, pady = 5, sticky = "nsew")
        ttk.Button(self.frame, text = "abbrechen", command = self.abbruch).grid(
                    column = 2, row = 3, padx = 5, pady = 5, sticky = "nsew")

    def ende(self):
        """ Beendet das Einstellungsfenster,
            sichert die geänderten Werte in den Klassenvariablen
            und schreibt die Werte in die Module
            Sichert auch die Werte des anderen Registers
            """
        self.write_Values()       
        __class__.put_Values()
        
        TX_Register.write_Values(TX_Register.my_self)
        TX_Register.put_Values()

        self.top_level.destroy()
        
    def write_Values(self):
        """ alle Objektvariablen in die Klassenvariablen schreiben """
        for i in range(self.len_items):
            __class__.chk_var[i] = self.chk_var[i].get()

    def abbruch(self):
        """ Beendet das Widget ohne Änderungen zu speichern """
        #__class__.get_Values()
        self.top_level.destroy()
        
    def dummy(self):       
        """ Ohne einen Funktionsaufruf in Checkbutton() werden dort
            keine Werte angezeigt
        """
        pass

class TX_Register():
    
    """ In ein bestehendes Widget werden beliebig viele Checkboxen zur Auswahl
        von Einstellungen plaziert.
        Die Einstellungen werden in Klassenvariablen gespeichert.
        Die Checkbox erhalten aufsteigende "onvalues".
        Die gesamte Einstellung wird in einer Int-Variablen übergeben,
        in der jede Checkbox durch ein Bit repräsentiert wird.
    """

    # Tuple aus Tuplen("Bezeichnung des Buttons", Vorgabewert, onvalue, Column, Row)
    items = (("0x08 AT-Cmd",               0, 0x01, 0, 0),
             ("0x17 Remote AT",            0, 0x02, 0, 1),
             ("0x10 Data Frame",           0, 0x04, 0, 2),
             ("Disable Ack",               0, 0x01, 0, 0),
             ("Apply Changes on rem.",     1, 0x02, 0, 1),
             ("Ext. Transm. Timeout",      0, 0x40, 0, 2),
             ("Disable Ack",               0, 0x01, 0, 0),
             ("Enable APS encryption",     0, 0x20, 0, 1),
             ("Ext. Transm. Timeout",      0, 0x40, 0, 2))

    # für jede Checkbox eine Variable für den aktuellen Wert anlegen 
    chk_var = [None] * len(items)

    my_self = None
    # die Variablen mit den Vorgabewerten (onvalue oder 0) aus den "items" initialisieren
    for i in range(len(items)):
        if items[i][1] > 0:
            chk_var[i] = items[i][2] 
        else:
            chk_var[i] = 0

    radio_var = 1
    radius_var = 0
    
    def put_Values():
        """ Schreibt die in den Klassenvariablen gespeicherten Konfigurationswerte
            in die zuständigen Module
        """
        at_show = 0
        for i in range(3):
            at_show += __class__.chk_var[i]
        
        at_options = 0
        for i in range(3,6):
            at_options += __class__.chk_var[i]       

        data_options = 0
        for i in range(6,9):
            data_options += __class__.chk_var[i]

#        print("AT-Show {}, AT-Options {}, Data_Options {}, Frame_ID {}, Broadcast_Radius {}".format(
#            at_show, at_options, data_options, __class__.radio_var, __class__.radius_var))

        inputs.show_frame = at_show
        inputs.Input_0x10.tx_request_opt = data_options
        inputs.Input_0x17.remote_cmd_opt = at_options
        inputs.set_Frame_Id(__class__.radio_var)

    def __init__(self, master, top_level = None, width = 18):
        
        self.top_level = top_level
        __class__.my_self = self
        
        self.len_items = len(__class__.items)
        self.chk_var = [None] * self.len_items

        frm_tx = ttk.LabelFrame(master, text = "Show Tx-Frames")
        frm_tx.grid(column = 0, row =0, padx = 5, pady =5, rowspan = 4, sticky = "nsew")       
        
        for i in range(3):
            self.chk_var[i] = Tk.IntVar()
            self.chk_var[i].set(self.__class__.chk_var[i])
            
            ttk.Checkbutton(frm_tx, text = __class__.items[i][0],
                            variable = self.chk_var[i],
                            width = width,
                            onvalue = __class__.items[i][2],
                            command = self.dummy).grid(
                            column = __class__.items[i][3],
                            row = __class__.items[i][4],
                            padx = 5,
                            pady = 5,
                            sticky = "nsw")

        frm_radio = ttk.LabelFrame(master, text = "Frame ID")
        frm_radio.grid(column = 1, row = 0, padx = 5, pady =5, rowspan = 2, sticky = "nsew")

        self.__radio = Tk.IntVar()
        self.__radio.set(__class__.radio_var)
        self.__rd1 = ttk.Radiobutton(frm_radio, variable = self.__radio, value = 1,
                                    text = "ID fortlaufend", width = width)
        self.__rd2 = ttk.Radiobutton(frm_radio, variable = self.__radio, value = 0,
                                    text = "ID fest ==  0")        
        self.__rd1.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "w")
        self.__rd2.grid(column = 0, row = 1, padx = 5, pady = 5, sticky = "w")

        frm_spin = ttk.LabelFrame(master, text = "Broadcast Radius")
        frm_spin.grid(column = 1, row = 2, padx = 5, pady =5, rowspan = 2,sticky = "nsew")

        self.__radius = Tk.IntVar()
        self.__radius.set(__class__.radius_var)
        spibx = Tk.Spinbox(frm_spin, width = 4, textvariable = self.__radius,
                           from_ = 0, to = 20, state = "readonly")
        spibx.grid(column = 0, row = 1, padx = 4, pady = 3, sticky = "ew")
        ttk.Label(frm_spin, text = "Frame 0x10").grid(column = 1,row = 1, padx = 5, pady = 5, sticky ="nsw")

        frm_rco = ttk.LabelFrame(master, text = "Remote Cmd Options")
        frm_rco.grid(column = 2, row = 0, padx = 5, pady =5, rowspan = 3, sticky = "nsew")

        for i in range(3, 6):
            self.chk_var[i] = Tk.IntVar()
            self.chk_var[i].set(self.__class__.chk_var[i])
            
            ttk.Checkbutton(frm_rco, text = __class__.items[i][0],
                            variable = self.chk_var[i],
                            width = width,
                            onvalue = __class__.items[i][2],
                            command = self.dummy).grid(
                            column = __class__.items[i][3],
                            row = __class__.items[i][4],
                            padx = 5,
                            pady = 5,
                            sticky = "nsw")

        frm_do = ttk.LabelFrame(master, text = "Data Options")
        frm_do.grid(column = 3, row = 0, padx = 5, pady =5, rowspan = 3,sticky = "nsew")

        for i in range(6, 9):
            self.chk_var[i] = Tk.IntVar()
            self.chk_var[i].set(self.__class__.chk_var[i])
            
            ttk.Checkbutton(frm_do, text = __class__.items[i][0],
                            variable = self.chk_var[i],
                            width = width,
                            onvalue = __class__.items[i][2],
                            command = self.dummy).grid(
                            column = __class__.items[i][3],
                            row = __class__.items[i][4],
                            padx = 5,
                            pady = 5,
                            sticky = "nsw")

        ttk.Button(master, text = "übernehmen", command = self.ende).grid(
            column = 2, row = 3, padx = 5, pady = 5, sticky = "nsew")
        ttk.Button(master, text = "abbrechen", command = self.abbruch).grid(
            column = 3, row = 3, padx = 5, pady = 5, sticky = "nsew")  
        
    def ende(self):
        
        self.write_Values()
        self.__class__.put_Values()

        RX_Register.write_Values(RX_Register.my_self)       
        RX_Register.put_Values()
        self.top_level.destroy()

    def write_Values(self):       
        """ alle Variablen auslesen und in die Klassenvariablen schreiben """
        for i in range(self.len_items):
            __class__.chk_var[i] = self.chk_var[i].get()
        __class__.radius_var = self.__radius.get() 
        __class__.radio_var = self.__radio.get() 

    def abbruch(self):
        """ Abbruch ohne übernahme der Änderungen """
        #self.__class__.get_Values()
        self.top_level.destroy()
        
    def dummy(self):
        
        """ Ohne einen Funktionsaufruf in Checkbutton() werden dort keine Werte angezeigt """
        pass          

        

def einstellungen(parent):

    w = Tk.Toplevel()       # ein unabhängiges Toplevel-Window öffnen
    w.resizable(width = False, height = False) # Größe nicht veränderbar
    w.title("Einstellungen")
    w.geometry('+200+150')


    nb = ttk.Notebook(w, width=640, height=160)

    reg1 = ttk.Frame(nb)
    reg2 = ttk.Frame(nb)

    nb.add(reg1, text=' Transmit Options ',   padding = 5, sticky = "ewns")    
    nb.add(reg2, text=' Receive Options ',    padding = 5, sticky = "ewns")

    nb.grid(column = 0, row = 0)
    TX_Register(reg1, top_level = w)
    RX_Register(reg2, top_level = w)

    w.focus_set()           # focus auf das Fenster
    w.grab_set()            # alle Events werden nur nach w durchgereicht,
                            # das drunterliegende Fenster bleibt inaktiv, wird aber aktualisiert
    w.transient(parent)     # das Fenster bleibt oberhalb von Top
#   w.wait_window(w)        # warten bis zum Beenden des Fensters, aber ...
                            # ... in der Zeit arbeiten andere Threads nicht weiter
    w.mainloop()            # warten bis zum Beenden des Fensters



if __name__ == "__main__":

    def start():
        einstellungen(parent = top)

    top = Tk.Tk()
    frm = Tk.LabelFrame(top, text = "Toplevel-Test")
    frm.grid(column = 0, row = 0)
    lbl = Tk.Button(frm, text = " Config ",  width = 20, command = start)
    lbl.grid(column = 0, row = 0, sticky ="nswe") 

    top.mainloop()
