# --- [ test_inputs.py ] ------------------------------------------------------------------------------
#
# Hier werden die 3 unterschiedlich Eingabezeilen als Klassen definiert

import tkinter as Tk
from tkinter import ttk
import _thread
import time

import check
import cmd_line
import xbee_adr
import tools
import cl_combo
import xbee_adr

file_handle = None      # für das Loggen in eine Datei
tm_dez   = False        # true: Die Zeit mit 2 Nachkommastellen für Sekunden ausgeben
comport  = None         # Pointer auf die Serielle Schnittstelle
tx1      = None         # Editor für Textausgabe
tx2      = None

show_frame = 0          # Bitmaske, steuert die Anzeige von Frames


def format_time():
    """ Einheitliche Formatierung für die Sendezeit ggf. incl. Nachkommastellen"""
    t = time.ctime().split()[3]

    if tm_dez:
        t += '.' + "{:>02}".format(str(int(round((time.time() % 1),2) * 100)))
    return t

def set_Frame_Id(val):
    """ Setzt die Frame_Id in den Eingabeobjekten auf 0 oder 1
        wenn Frame_Id > 0, dann wird vor jeder Sendung der Wert incrementiert.
        Bei Frame ID == 0 wird keine Bestätigung vom Empfänger zurückgesendet.
    """
    
    if Input_0x08.frame_id == 0 and val: 
        Input_0x08.frame_id = 1
        Input_0x10.frame_id = 1
        Input_0x17.frame_id = 1

    elif Input_0x08.frame_id and val == 0:  
        Input_0x08.frame_id = 0
        Input_0x10.frame_id = 0
        Input_0x17.frame_id = 0

def get_next_Frame_Id():
    """ Incrementiert die Frame_Id und liefert den Wert zurück
        jedoch nur bis zum Wert 255
        und dann nicht, wenn frame_id == 0
    """
    i = Input_0x08.frame_id
    i += 1
    if i > 255:
        i = 1
    Input_0x08.frame_id = i
    return i


### --- FRAME0x08 --- Local AT -------------------------------------------------

class Input_0x08(object):
    
    """ Ist zuständig für die Eingabe und das Absenden von Lokalen AT-Frames.
        Enthalten sind ein erläuterndes Label, ein Entry-Feld für die Texteingabe und
        zwei Buttons (zum Senden und Löschen des Eingabefeldes)
        Auch ein Return im Entry-Feld sendet den Eingabestring.
    """ 
     
    frame_id   = 1          # getrennte Frame_ID#s für Frame 0x08, 0x10 und 0x17
 #   frame_id_x = 1          # für "verdeckt" gesendete Frames
    
    def __init__(self, master,
                 row = 0,
                 column = 0
                 ):
        # Regelt den Abstand zum linken Rand
        ttk.Label(master, width = 23).grid(column = column, row = row, padx = 5, pady = 5)

        self.__lbl = ttk.Label(master, text = "Local AT-CMD:", width = 15, anchor = "e" )
        self.__lbl.grid(column = column + 1, row = row, padx = 5, pady = 5, sticky = "e")

        self.__btn1 = ttk.Button(master, width = 8, text = "Clr Cmd", command = self.__clr)
        self.__btn1.grid(column = column + 2, row = row, padx = 5, pady = 5, sticky = "e")

        self.__entry = ttk.Entry(master, width = 24, font = ('Courier', 10, 'bold'))
        self.__entry.grid(column = column + 3, row = row, padx = 5, pady = 5, sticky = "nsew")
        self.__variable = Tk.StringVar()
        self.__entry.config(textvariable = self.__variable)
        self.__entry.bind(sequence ='<Return>', func= self.__send) 

        self.__btn2 = ttk.Button(master, width = 8, text = "Send Cmd", command = self.__send)
        self.__btn2.grid(column = column + 4, row = row, padx = 5, pady = 5, sticky = "e")

    
    def __clr(self):
        self.__variable.set("")
        self.__entry.focus_set()
                
    def __send(self, event = None):
        at, data = check_cmd(self.__variable.get())
        if at:
            self.__frm08_send( at, data )
        else:
            self.__lbl.bell()
        self.__entry.focus_set()

    def get_next_Frame_Id(self):
        """ Liefert die nächste Frame_ID,
            sofern Frame_Id nicht == 0
        """
        if __class__.frame_id != 0:               # wenn Frame_ID == 0, dann nichts tun
            __class__.frame_id += 1               # Frame_ID hochzählen
            if __class__.frame_id > 255:
                __class__.frame_id = 1            # die Null dabei überspringen
        return __class__.frame_id


    def __frm08_send(self, at, data):   # Parameter: Tuple: AT-CMD, Parameter als bArray

        cmd = bytearray()
        cmd.append(0x7E)                # Start Delimter
        cmd.append(0)                   # high Byte Länge
        cmd.append(0)                   # Platzhahlter für low Byte Länge
        cmd.append(8)                   # Frame-Typ
        cmd.append(self.get_next_Frame_Id())     
        cmd.append(ord(at[0]))          # ATCMD
        cmd.append(ord(at[1]))          # ATCMD
        for i in range(len(data)):
            cmd.append(data[i])

        l, c = check.check_sum(cmd)     # Checksum und Länge bestimmen und anhängen
        cmd[2] = l                      # Länge im dritten Byte einfügen
        cmd.append(c)                   # checksumme anhängen

        if not comport == None:
            comport.write(cmd)          # senden

        datastr = ""                    # Druckausgabe vorbereiten 
        for i in range(len(data)):
            datastr += "{:>02X}".format(data[i])
        
        if datastr:                     # Data weglassen, wenn es keine Parameter gibt
            t = "< {},{},{},{},{}"
        else:
            t = "< {},{},{},{}"
        
        t1 = t.format(format_time(), "08",
                  "{:>02X}".format(self.frame_id), at, datastr)

        tx1.insert('end', t1 +"\n")     # an den Editor senden
        tx1.see('end')
        
        if not file_handle == None:
            file_handle.write(t1 + "\n")
        
        if not show_frame & 0x01:
            return
                   
        msg = []
        msg.append("<- FRAME 0x08 - Local AT Command")
        msg.append("Frame ID      : " +"{:>02X}".format(self.frame_id))
        msg.append("AT Command    : " + at)

        if len(datastr):
            data_hex = ""
            data_dez = ""
            data_asc = ""

            for i in range(0, len(datastr), 2):
                c = int(datastr[i:i+2], 16)
                data_hex += "  " + datastr[i:i+2] 
                data_dez += " " + "{:>3}".format(c)
                if c > 32 and c < 128:
                    data_asc += "   " + chr(c)
                else:
                    data_asc += "    "
            msg.append("Data [hex]    : " + data_hex)
            msg.append("Data [dez]    : " + data_dez)
            msg.append("Data [asc]    : " + data_asc) 

        tx2.insert('end', "\n")
        for i in msg:
            tx2.insert('end', i + "\n")
        tx2.see('end')


### --- FRAME0x17 --- Remote AT -------------------------------------------------

class Input_0x17(object):
    
    """ Ist zuständig für die Eingabe und das Absenden von Remote AT-Frames.
        Enthalten sind ein erläuterndes Label,eine Listbox zur Auswahl des Zieles,
        ein Entry-Feld für die Texteingabe und
        zwei Buttons (zum Senden und Löschen des Eingabefeldes)
        Auch ein Return im Entry-Feld sendet den Eingabestring.
    """  
    
    frame_id       = 1          # getrennt Frame_ID#s für Frame 0x08, 0x10  und 0x17
    remote_cmd_opt = 2

    combos = []

    def update_Combos(new_list):
        for i in __class__.combos:
            i.change_list(new_list)
    
    def get_next_Frame_Id(self):
        """ Liefert die nächste Frame_ID,
            sofern Frame_Id nicht == 0
        """
        if __class__.frame_id != 0:               # wenn Frame_ID == 0, dann nichts tun
            __class__.frame_id += 1               # Frame_ID hochzählen
            if __class__.frame_id > 255:
                __class__.frame_id = 1            # die Null dabei überspringen
        return __class__.frame_id


    def __init__(self, master,
                 column = 0,
                 row = 0,
                 optionlist = None
                 ):
 
        self.__lbl = ttk.Label(master, text = "Remote AT-CMD:", width = 15, anchor = "e")
        self.__lbl.grid(column = column, row = row, padx = 5, pady = 5, sticky = "e")

        self.om = cl_combo.Combo(master, column = column + 1, row = row)
        __class__.combos.append(self.om)     

        self.__btn1 = ttk.Button(master, width = 8, text = "Clr Cmd", command = self.__clr)
        self.__btn1.grid(column = column + 2, row = row, padx = 5, pady = 5, sticky = "e") 

        self.__entry = ttk.Entry(master, width = 24, font = ('Courier', 10, 'bold' ))
        self.__entry.grid(column = column + 3, row = row, padx = 5, pady = 5, sticky = "nswe")
        self.__variable = Tk.StringVar()
        self.__entry.config(textvariable = self.__variable)
        self.__entry.bind(sequence ='<Return>', func= self.__send) 

        self.__btn2 = ttk.Button(master, width = 8, text = "Send Cmd", command = self.__send)
        self.__btn2.grid(column = column + 4, row = row, padx = 5, pady = 5, sticky = "e")

        
    def __clr(self):
        self.__variable.set("")
        self.__entry.focus_set()
                
    def __send(self, event = None):
        at, data = check_cmd(self.__variable.get())
        if at:
            self.__frm17_send(at, data, self.om.get_Idx())
        else:
            self.__btn1.bell()
        self.__entry.focus_set()



    def __frm17_send(self, at, data, dev):# Parameter: Tuple: AT-CMD, Parameter als bArray, device als Index
        
        cmd = bytearray()
        cmd.append(0x7E)                # Start Delimter
        cmd.append(0)                   # high Byte Länge
        cmd.append(0)                   # Platzhahlter für low Byte Länge
        cmd.append(0x17)                # Frame-Typ
        cmd.append(self.get_next_Frame_Id()) 

        adr_64bit = ""          
        for i in range(8):              # die 64-bit Destination Adresse
            adr_64bit += "{:>02X}".format(xbee_adr.adr_64bit[dev][i])
            cmd.append(xbee_adr.adr_64bit[dev][i])

        adr_16bit = ""
        for i in range(2):              # die 16-bit Destination Network Adresse
            adr_16bit += "{:>02X}".format(xbee_adr.adr_16bit[dev][i])
            cmd.append(xbee_adr.adr_16bit[dev][i])
        
        cmd.append(self.remote_cmd_opt) # Options : 2 = Apply Changes
        cmd.append(ord(at[0]))          # ATCMD
        cmd.append(ord(at[1]))          # ATCMD

        datastr = ""                    # Druckausgabe vorbereiten
        for i in range(len(data)):
            datastr += "{:>02X}".format(data[i])
            cmd.append(data[i])
    
        l, c = check.check_sum(cmd)     # Checksum und Länge bestimmen und anhängen
        cmd[2] = l                      # Länge im dritten Byte einfügen
        cmd.append(c)                   # checksumme anhängen

        if not comport == None:
            comport.write(cmd)


        if datastr:                     # "Data" weglassen, wenn keine Daten vorhanden
            t = "< {},{},{},{},{},{},{},{}"
        else:
            t = "< {},{},{},{},{},{},{}"
        
        _frame_id = "{:>02X}".format(self.frame_id)
        cmd_options = "{:>02X}".format(self.remote_cmd_opt)
        
        t1 = t.format(format_time(), "17",
              _frame_id,
              adr_64bit, adr_16bit, cmd_options,
              at, datastr)

        tx1.insert('end', t1 +"\n")     # an den Editor senden
        tx1.see('end')

        if not file_handle == None:
            file_handle.write(t1 + "\n")
            
        if not show_frame & 0x02:
            return
        
        msg = []
        msg.append("<- FRAME 0x17 - Remote AT Command")
        msg.append("Frame ID      : " + _frame_id)
        msg.append("Destin.64-Bit : " + adr_64bit)
        msg.append("Destin.16-Bit : " + adr_16bit)
        options = tools.get_bits_in_dict(tx_options, self.remote_cmd_opt)
        if len(options) == 0:   # nichts zutreffendes gefunden
            msg.append("RemoteCmd Opt.: " + cmd_options)
        else:
            msg.append("RemoteCmd Opt.: " + cmd_options + "-" + options[0])
            for i in range(1, len(options)):
                msg.append("              : " + cmd_options + "-" + options[i])
        msg.append("At-Command    : " + at)

        if len(datastr):
            data_hex = ""
            data_dez = ""
            data_asc = ""

            for i in range(0, len(datastr), 2):
                c = int(datastr[i:i+2], 16)
                data_hex += "  " + datastr[i:i+2] 
                data_dez += " " + "{:>3}".format(c)
                if c > 32 and c < 128:
                    data_asc += "   " + chr(c)
                else:
                    data_asc += "    "

            msg.append("Data [hex]    : " + data_hex)
            msg.append("Data [dez]    : " + data_dez)
            msg.append("Data [asc]    : " + data_asc)

        tx2.insert('end', "\n")
        for i in msg:
            tx2.insert('end', i + "\n")
        tx2.see('end')


### --- FRAME0x10 --- ZigBee Transmit Request -------------------------------------
        
class Input_0x10(object):
    """ Ist zuständig für die Eingabe und das Absenden von Data AT-Frames.
        Enthalten sind ein erläuterndes Label, eine Listbox zur Auswahl des Zieles,
        ein Entry-Feld für die Texteingabe und
        zwei Buttons (zum Senden und Löschen des Eingabefeldes)
        Auch ein Return im Entry-Feld sendet den Eingabestring.
    """ 
    
    frame_id       = 1        # getrennt Frame_ID#s für Frame 0x08, 0x10 und 0x17
    tx_request_opt = 0
    broadcast_radius = 0
    
    combos = []

    def update_Combos(new_list):
        for i in __class__.combos:
            i.change_list(new_list)

    def get_next_Frame_Id(self):
        """ Liefert die nächste Frame_ID,
            sofern Frame_Id nicht == 0
        """
        if __class__.frame_id != 0:               # wenn Frame_ID == 0, dann nichts tun
            __class__.frame_id += 1               # Frame_ID hochzählen
            if __class__.frame_id > 255:
                __class__.frame_id = 1            # die Null dabei überspringen
        return __class__.frame_id
    
    def __init__(self, master,
                 column = 0,
                 row = 0,
                 optionlist = None
                ):
 
        self.__lbl = ttk.Label(master, text = "Data Frame:", width = 15, anchor = "e")
        self.__lbl.grid(column = column, row = row, padx = 5, pady = 5, sticky = "e")

        self.om = cl_combo.Combo(master, column = column + 1, row = row)
        __class__.combos.append(self.om)
        
        self.__btn1 = ttk.Button(master, width = 8, text = "Clr Data", command = self.__clr)
        self.__btn1.grid(column = column + 2, row = row, padx = 5, pady = 5, sticky = "e")
        
        self.__entry = ttk.Entry(master, width = 24, font = ('Courier', 10, 'bold'))
        self.__entry.grid(column = column + 3, row = row, padx = 5, pady = 5, sticky = "nswe")
        self.__variable = Tk.StringVar()
        self.__entry.config(textvariable = self.__variable)
        self.__entry.bind(sequence ='<Return>', func= self.__send) 

        self.__btn2 = ttk.Button(master, width = 8, text = "Send Data", command = self.__send)
        self.__btn2.grid(column = column + 4, row = row, padx = 5, pady = 5, sticky = "e")

        
    def __clr(self):
        self.__variable.set("")
        self.__entry.focus_set()
                
    def __send(self, event = None):
        chk = cmd_line.split_string(self.__variable.get())
        if chk:
            self.__frm10_send(chk, self.om.get_Idx())
        else:
            self.__om.bell()
        self.__entry.focus_set()


    def __frm10_send(self, data, dev):  # Parameter: AT-CMD, Parameter als bArray
        
        cmd = bytearray()
        cmd.append(0x7E)                # Start Delimter
        cmd.append(0)                   # high Byte Länge
        cmd.append(0)                   # Platzhahlter für low Byte Länge
        cmd.append(0x10)                # Frame-Typ
        cmd.append(self.get_next_Frame_Id())

        adr_64bit = ""
        for i in range(8):              # die 64-bit Destination Adresse
            adr_64bit += "{:>02X}".format(xbee_adr.adr_64bit[dev][i])
            cmd.append(xbee_adr.adr_64bit[dev][i])

        adr_16bit = ""
        for i in range(2):              # die 16-bit Destination Network Adresse
            adr_16bit += "{:>02X}".format(xbee_adr.adr_16bit[dev][i])
            cmd.append(xbee_adr.adr_16bit[dev][i])

        cmd.append(self.broadcast_radius)# Broadcast Radius
        cmd.append(self.tx_request_opt) # Options

        datastr = ""
        for i in range(len(data)):      # die Parameter
            datastr += "{:>02X}".format(data[i])
            cmd.append(data[i])
    
        l, c = check.check_sum(cmd)     # Checksum und Länge bestimmen und anhängen
        cmd[2] = l                      # Länge einfügen
        cmd.append(c)                   # checksumme anhängen

        if not comport == None:
            comport.write(cmd)          # senden

        _frame_id = "{:>02X}".format(self.frame_id)
        _broadcast_radius = "{:>02X}".format(self.broadcast_radius)
        _options          = "{:>02X}".format(self.tx_request_opt)
        
        t = "< {},{},{},{},{},{},{},{}"
        t1 = t.format(format_time(), "10", _frame_id,
              adr_64bit, adr_16bit,
              _broadcast_radius, _options, datastr)

        tx1.insert('end', t1 +"\n")     # an den Editor senden
        tx1.see('end')

        if not file_handle == None:
            file_handle.write(t1 + "\n")
            
        if not show_frame & 0x04:
            return
        
        msg = []
        msg.append("<- FRAME 0x10 - ZigBee Transmit Request")
        msg.append("Frame ID      : " + _frame_id)
        msg.append("Destin.64-Bit : " + adr_64bit)
        msg.append("Destin.16-Bit : " + adr_16bit)
        msg.append("Broadcast Rad.: " + _broadcast_radius)

        options = tools.get_bits_in_dict(tx_options, self.tx_request_opt)
        if len(options) == 0:
            msg.append("Options       : " + _options)
        else:
            msg.append("Options       : " + _options + "-" + options[0])
            for i in range(1, len(options)):
                msg.append("              : " + _options + "-" + options[i])

        if len(datastr):
            data_hex = ""
            data_dez = ""
            data_asc = ""

            for i in range(0, len(datastr), 2):
                c = int(datastr[i:i+2], 16)
                data_hex += "  " + datastr[i:i+2] 
                data_dez += " " + "{:>3}".format(c)
                if c > 32 and c < 128:
                    data_asc += "   " + chr(c)
                else:
                    data_asc += "    "

            msg.append("Data [hex]    : " + data_hex)
            msg.append("Data [dez]    : " + data_dez)
            msg.append("Data [asc]    : " + data_asc)

        tx2.insert('end', "\n")
        for i in msg:
            tx2.insert('end', i + "\n")
        tx2.see('end')


class Send_At_Cmd(object):
    """ Sendet "verdeckt" (ohne Ausgabe im Log) das AT-Command "DB" um die
        Signalstärke des letzten empfangenen Frames auszulesen.
        Führt eine eigene Frame_Id mit, damit auch bei der Grundeinstellung
        Frame_ID == 0 das Modem abgefragt werden kann.
    """
    def __init__(self):
        self.frame_id = 1

    def cmd_DB(self):
        self.frame_id += 1
        if self.frame_id > 255:
            self.frame_id = 1
           
        cmd = [ 0x7E,0, 0, 8, self.frame_id, ord("D"), ord("B")]
        l, c = check.check_sum(cmd)     # Checksum und Länge bestimmen

        cmd.append(c)
        cmd[2] = l

        if not comport == None:
            comport.write(cmd)
            return self.frame_id
        else:
            return 0


# --- den Eingabestring testen -------------------------------------------------

def check_cmd(cmd):
    """ Einen Eingabestring überprüfen:
        1.) ist der String AT vorangestellt, wenn ja, dann abschneiden
        2.) sind die beiden ersten Char im Dictonary der AT-Command eingetragen
        3.) die Parameter analysieren und - wenn möglich - in Bytes umwandeln

        Rückgabe:
        1. Wert: AT-Command in Großbuchtaben - bei Fehler ""
        2. Wert: AT-Command in Langtext - bei Fehler ""
    """

    if cmd[:2].upper() == "AT":     # optionales "AT" abschneiden
        cmd = cmd[2:]
    
    at_cmd = check.check_atcmd(cmd[:2]) # liefert Tuple (cmd.upper(),Langtext AT-CMD)
    if at_cmd[0]:                       # Kommando, wenn > ""
        b = cmd_line.split_string(cmd[2:])# Parameter hinter dem AT-CMD in Bytes umwandeln
    else:
        b = ""
    return at_cmd[0], b
        

tx_options = {          # werden in Frame0x10 und 0x17 benutzt
            0x01 : "Disable ACK",
            0x02 : "Apply changes on remote",
            0x20 : "Enable APS encryption",
            0x40 : "Extended transmisson timeout"}
  

if __name__ == "__main__":

    class app(object):  

        def __init__(self):
            f = open("xbee.cfg")
            xbee_adr.read_adr(f)
            f.close()
                     
            top = Tk.Tk()
            s = ttk.Style()
            s.configure('.', font = ('Arial', 8))
            
            self.frm1 = ttk.LabelFrame(top, text = "TESTstring")
            self.frm1.grid(column = 0, row = 0, padx = 5, pady = 3)

            self.cmd1 = Input_0x08(self.frm1, column = 0, row = 0)
            self.cmd2 = Input_0x17(self.frm1, column = 0, row = 1, optionlist= (xbee_adr.adr_names))
            self.cmd3 = Input_0x10(self.frm1, column = 0, row = 2, optionlist= (xbee_adr.adr_names))

            top.mainloop()

    app()

# --- [ eof ] ------------------------------------------------------------------
