# --- [ read_cfg.py ] -------------------------------------------------------
#
# Ist zuständig für das Einlesen der Datei "xbee.cfg"
# Dort sind die Adressen der Stationen sowie Parameter abgelegt.
# Die Comports werden abgefragt.

import serial
import xbee_adr    

def read_cfg():
    """ Öffnet die Datei "adr.cfg" und liest die XBee-Adr ein """
    try:
        f = open("xbee.cfg")
    except:
        print("Die Datei 'xbee.cfg' kann nicht geöffnet werden")
        return
        

    # die XBee-Adr müssen am Anfang der Datei stehen
    xbee_adr.read_adr(f)

    # von nun an gehts zeilenweise weiter
    default_comport = None
    comport = None
    width = 10
    height = 20
    
    z = f.readline()
    while z:

        if z[0:1] == "#":       # Zeilen, die mit '#' beginnen sind Kommentare
            z = f.readline()
            continue

        a = z.split(":")        # Trennzeichen zwischen Schlüssel und Wert

        if len(a) != 2:         # wenn nicht genau EIN Trennzeichen enthalten war
            z = f.readline()
            continue
        
        key = a[0].strip()
        value = a[1].strip()

        #print(key, value)

        if key == "COMPORT":
            default_comport = value

        if key == "TEXT_WIDTH":
            width = int(value)

        if key == "TEXT_HEIGHT":
            height = int(value)

        if key == "TIME_DEZ":
            time_dez = int(value)

        z = f.readline()

    f.close()

    ports = seek_Free_Serial_Ports()

    print("Freie ComPorts: ", ports)

    if len(ports) == 1:     # genau ein freier Port gefunden
        comport = serial.Serial(ports[0], 9600)
        print("geöffnet :", comport.name)

    elif len(ports) == 0:   # kein freier Port gefunden
        print("kein ComPort gefunden !")

    else:                   # mehrere freie Ports gefunden
        if default_comport in ports:
            comport = serial.Serial(default_comport, 9600)
            print("geöffnet : ", default_comport)
    
        else:
            print("welcher ComPort soll geöffnet werden ?")
            exit()

    #print(comport, width, height, time_dez)
    return (comport, width, height, time_dez)

 
# ----------------------------------------------------------------
def seek_Free_Serial_Ports(z = 9):
    """Prüft, ob sich die Ports COM1: bis COM10: öffnen lassen
       gibt eine Liste der öffenbaren Ports zurück.
    """
    port_list = []

# Die Ports können als "COM1" .. "COM10" oder
# über eine Ziffer      0     ..  9 angesprochen werden

    for i in range(1, z):
#    for i in range(0, z):
        try:
            p  = "COM" + str(i)
            rx = serial.Serial(p)
#            rx = serial.Serial(i)
            rx.close()
            port_list.append(p)
#            port_list.append(i)
        except:
            pass

    return port_list
##
##"""
##    serial.Serial(Port, Baudrate)
##    close()
##    open()
##    setTimeout(x)
##    getTimeout()  / timeout
##    setBaudrate(x)
##    getBaudrate() / baudrate
##    setDTR(x)
##    setRTS(x)
##    getCTS()
##    getDSR()
##
##    inWaiting()
##    read(x)
##    readline()
##    flushinput()
##    flushoutput()
##    flush()
##    write(data.encode(encoding = 'utf-8'))
##
##"""


if __name__ == "__main__":
    
    print(read_cfg())
    
# --- [ eof ] ------------------------------------------------------------------
