# --- [ receive.py ] -----------------------------------------------------------
#
# Ist für das Empfangen von Frames über der Seriellen Schnittstelle zuständig
# Prüft die Checksumme und leitet zur Darstellung weiter an "frames.py"

import serial
import time
import frames
import xbee_adr
import inputs

show_frame = 0x200                      # ein Flag welches angibt, ob automatisch die
                                        # Signalstärke ausgelesen wird

tm_dez = False                          # true: Die Zeit mit 2 Nachkommastellen ausgeben
                                        # die Voreinstellung steht in  "xbee.cfg"

def Serial_Read_Thread(comport):
    """ Das Auslesen der Seriellen Schnittstelle läuft
        in einem eigenen Thread im Hintergrund ab 

        Wenn mindestens 3 Byte an Daten anstehen, dann werden diese eingelesen
        Es sollte sich um die ersten drei Bytes eines Frames halten,
        daher mit 0x7E und 0x00 beginnen, nachfolgend das Byte mit der Länges des Datensatzes
        das muss zuallerst analysiert werden.
        Danach wird auf den Rest des Datensatzes gewartet.
        Ist er komplett, dann wird die Prüfsumme getestet.
        Ist diese korrekt, dann kann der Datensatz an das nächste Modul übergeben werden
    """
    if comport == None:                 # wenn kein Comport gefunden wurde,
        while True:                     # dann einfach nur abwarten ....
            time.sleep(1000)
            
    comport.flush()                     # alle anstehenden Daten löschen

    job_list = []
    source = 0
    Send_Cmd = inputs.Send_At_Cmd()
    
    while True:
        i = comport.inWaiting()
        if i >= 3:
            if tm_dez:                  # aber erst später an frm_time anhängen          
                frm_time_dez = "{:>02}".format(str(int(round((time.time() % 1),2) * 100)))

            frm_time = time.ctime().split()[3]

            frm_head = comport.read(3)  # die ersten 3 Bytes einlesen

            # Wenn die ersten beiden Bytes nicht mit dem erwarteten Beginn eines Frames übereinstimmmen.
            # dann wurde kein Framestart erkannt, wir sind mitten in einem Frame gelandet.
            # Warten bis zum Ende der Übertragung und alle Daten in der Zwischenzeit verwerfen.
    
            if ( not (frm_head[0] == 0x7E and frm_head[1] == 0)):
                while True:
                    comport.flush()
                    time.sleep(.005)    # 5 Millisekunden, sollte für mindestens 1 Zeichen ausreichen
                    if comport.inWaiting() == 0: # wenn kein Byte eingegangen,
                        break           # dann kann die Schleife verlassen werden                        
            
            data_len = (frm_head[2] + 1)  # der verbleibende Rest des Frames mit den Daten

            # Warten bis das Frame vollständig im Empfangspuffer vorliegt

            while (comport.inWaiting() < data_len):
                time.sleep(.01)

            frm_data = comport.read(data_len)

            # Jetzt ist das komplette Frame eingelesen, nun die Prüfsummme testen
            # Die Summe des Datenbereiches bis incl. der Prüfsumme muss 255 ergeben

            chk = 0
            for i in range(len(frm_data)):
                chk += frm_data[i]
            
            if ((chk % 256) == 255):    # dann stimmt die Checksumme

                # genau 6 Zeichen mit der Zeit ohne ':' voranstellen,
                # dann läßt sich in frames die Positon einfach ermitteln (Faktor 2)
                hex_data = frm_time[0:2] + frm_time[3:5] + frm_time[6:]

                # den Frame ohne Header und Prüfsumme in Hexformat ohne führendes '0x' konvertieren
                for i in range(len(frm_data) - 1):
                    hex_data += "{:>02X}".format(frm_data[i])
                                   
                if tm_dez:
                    frm_time += "." + frm_time_dez  # die Nachkommastellen der Zeit anhängen

                # zur Behandlung der Frame-Typ verzweigen
                #print("Frame :", hex(frm_data[0]), " ",  len(frm_data))
                      
                if frm_data[0] == 0x88:
                    
                    # verdeckt gesendete AT-DB Commands zur Ermittlung des Empfangspegels
                    # sie stehen in der "job_list"
                    # die job_list darf nur ein Tuple enthalten
                    if len(job_list):
                        # wenn die Id's gleich sind und tatsächlich ein AT-DB gesendet wurde
                        if ((frm_data[1] == job_list[0][0]) and
                                (frm_data[2:4] == bytes("DB","ascii"))):
                                
                            # und Source nicht -1 = deaktiviert
                            if job_list[0][1] > 1:
                                xbee_adr.set_db(job_list[0][1], frm_data[5])

                            job_list = [] # Job_list löschen

                    else:
                        frames.frm88(frm_time, hex_data)

                elif frm_data[0] == 0x8A:
                    frames.frm8A(frm_time, hex_data)

                elif frm_data[0] == 0x8B:
                    # Auslesen der Empfangsstärke bei Frame 0x8B
                    # die Bestätigung des Frame0x10 liefert nur die 16Bit-Netzwerkadresse
                    if show_frame & 0x200:

                        # wenn bereits ein Eintrag in der Joblist vorhanden , aber noch
                        # nicht abgearbeitet ist, dann diesen Eintrag deaktivieren
                        if len(job_list):
                            # den Job 
                            job_list[0][1] = -1

                        else:
                            # die 16-Bit Netzwerkadresse suchen
                            adr = bytearray([frm_data[2], frm_data[3]])
                            source = xbee_adr.check_16bit_adr(adr)

                            # wenn die Adresse gefunden wurde
                            if source > 0:
                                id = Send_Cmd.cmd_DB()
                                job_list.append([id, source])

                        
                    frames.frm8B(frm_time, hex_data)

                elif frm_data[0] == 0x90:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frm90(frm_time, hex_data)

                elif frm_data[0] == 0x91:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frmXX(frm_time, hex_data) #

                elif frm_data[0] == 0x92:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frm92(frm_time, hex_data) #

                elif frm_data[0] == 0x94:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frmXX(frm_time, hex_data) #

                elif frm_data[0] == 0x95:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frm95(frm_time, hex_data) 
                    
                elif frm_data[0] == 0x97:
                    source = xbee_adr.check_xbee_adr(frm_data[2:10], frm_data[10:12])
                    frames.frm97(frm_time, hex_data)

                elif frm_data[0] == 0xA0:
                    frames.frmXX(frm_time, hex_data) #

                elif frm_data[0] == 0xA1: 
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frmA1(frm_time, hex_data) 

                elif frm_data[0] == 0xA3:
                    source = xbee_adr.check_xbee_adr(frm_data[1:9], frm_data[9:11])
                    frames.frmA3(frm_time, hex_data) 
                else:
                    raise ValueError("Unbekannter Frametyp :" + str(frm_data[0]))
                          
                # wenn automatisch die Empfangsstärke aus dem Modem ausgelesen werden soll
                # 'source' zeigt auf den Index des Absenders
                # oder auf 0, wenn der in der Liste Stationen nicht identifiziert werden kann
                
                if show_frame & 0x200 and source > 0:
                    # nicht in allen Frames ist der Absender in Frame erkennbar
                    if frm_data[0] in (0x90, 0x91, 0x92, 0x94, 0x95, 0x97, 0xA1, 0xA3):

                            # wenn bereits ein Job angelegt, aber nicht abgearbeitet
                            # ist, dann kann keine zusätzlicher Job zugeordnet werden.
                            # Durch setzen von source = -1 wird der Eintrag "entwertet"
                            if len(job_list):
                                job_list[0][1] = -1

                            else:
                                id = Send_Cmd.cmd_DB()
                                 # wenn id == 0, dann ist der Comport nicht geöffnet
                                if id: 
                                    job_list.append([id, source])                          

                            source = 0

            else:
                #raise ValueError("Frame Checksum Error")
                # Ein Fehler in der Checksum ist aufgetreten
                # die fehlerhaften Daten ausgeben
                print("Frame Checksum Error : ", end = " ")

                hex_data=""
                for i in range(len(frm_head)):
                    hex_data += "{:>02X}".format(frm_head[i])
                print(hex_data, end = " ")

                hex_data=""
                for i in range(len(frm_data) - 1):
                    hex_data += "{:>02X}".format(frm_data[i])
                print(hex_data)
                
                # falls noch Daten empfangen werden, alles verwerfen
                # bis eine Sende Pause erkannt wird
                while True:
                    comport.read(comport.inWaiting())
                    time.sleep(0.01)
                    if comport.inWaiting() == 0:
                        break
                print("flush")                        

            #  Stehen noch Daten eines weiteren Frames an ?
            #  Wenn ja, dann keine Wartezeit einlegen

            if (comport.inWaiting() > 0):
                continue
              
        time.sleep(0.1) # wenn keine Daten anstehen, dann eine Wartezeit einlegen



# --- [ eof ] ------------------------------------------------------------------
