# --- [ xbee_adr.py ] ----------------------------------------------------------
#
# Verwaltet die Daten der Stationen (Adressen, Namen)
# Überprüft beim Empfang von Daten, ob die gespeicherte 16-Bit Adresse
# einer Station mit der gespeicherten übereinstimmt.
# liest die 64-/16-Bit Adressen und Stationsnamen aus der
# Datei "xbee_adr.cfg" aus

import time
import inputs

adr_64bit = list()
adr_16bit = list()
adr_names = list()
adr_states = list()
adr_db = list()

tx = None

def check_xbee_adr(adr64, adr16):
    """ Prüft, ob die 64-Bit-Adresse in der Liste enthalten ist,
        wenn ja, dann wird geprüft, ob die 16-Bit-Netzwerkadresse
        mit der gespeicherten übereinstimmt,
        wenn nicht, dann wird die Adresse übernommen

        Ausgabe  ( 1, "Hinweistext") -> Adresse geändert
                 (-1, "Hinweistext") -> 64-Bit-Adresse nicht vorhanden """

    global adr_64bit, adr_16bit, adr_states
    
    result = -1
    
    for i in range(2, len(adr_64bit)):
       # print(adr_64bit[i], "-", adr64)
        if adr_64bit[i] == adr64:

            if adr_16bit[i] != adr16:
                oldadr = ""
                newadr = ""
                adr_64bit_str = ""
               
                for j in range(2):
                    oldadr += "{:>02X}".format(adr_16bit[i][j])
                    newadr += "{:>02X}".format(adr16[j])
                
                adr_16bit[i] = adr16

                for j in range(8):
                    adr_64bit_str += "{:>02X}".format(adr64[j])

                text = "  INFO : " + adr_64bit_str + " 16-Bit Adr changed: " + oldadr + " -> " + newadr

                if adr16 == bytearray([0,0]):
                    adr_states[0] = adr_names[i]        ## neben die Bezeichnung Coordinator seinen Namen schreiben
                    adr_states[i] = "Coordinator"
                
                elif adr16 == bytearray([0xFF, 0xFE]):
                    adr_states[i] = "offline"

                else:
                    adr_states[i] = "online"
                    
                result = 1
                break
            
            else:
                result = 0
                break

    if result == -1:
        text = "  INFO : Unbekannte Adresse"
        i = -1

    if result != 0:
        tx.insert('end', text + "\n")
        tx.see('end')

        # wenn sich Netzwerkadressen geändert haben, dann
        # die Inhalte Comboboxen aktualisieren.
        
        new_list = get_Station_Names()
        inputs.Input_0x17.update_Combos( new_list ) 
        inputs.Input_0x10.update_Combos( new_list ) 

    return i

def check_16bit_adr(adr):
    """ Wird ein Frame 0x8B empfangen, dann ist keine 64Bit Adresse
        enthalten, nur die 16-Bit Adresse.
        Hier wird die Station gesucht, zu der diese Adresse gehört.
        Falls die 16-Bit Adresse noch nicht bekannt ist,
        dann wird -1 zurückgegeben.
    """
    idx = -1
        
    for i in range(2, len(adr_16bit)):       
        if adr_16bit[i] == adr:
            idx = i
            break
    return idx
            

def read_adr(f):
    """ liest die 16Bit Adressen und die Namen der Stationen aus einer
        Konfigurationsdatei aus.
        Die 16Bit Adressen werden zunächst mit FFFE initialisert und später nach dem
        ersten Kontakt mit der Station mit der vergebenen Adresse überschrieben.
    """
    global adr_64bit, adr_16bit, adr_names, adr_states, adr_db
    
    zeile = f.readline()
    cnt = 0
    
    adr8 = [0,0,0,0,0,0,0,0]
    nw_adr = bytearray([0xFF, 0xFE])


    adr_64bit = list()
    adr_16bit = list()
    adr_names = list()
    adr_db = ["      ", "      "]

    adr_64bit.append(adr8)
    adr_16bit.append(bytearray([0,0]))
    adr_names.append("Coordinator")
    adr_states.append("?")

    adr8 = [0,0,0,0,0,0,0xFF,0xFF]
    adr_64bit.append(adr8)
    adr_16bit.append(nw_adr)
    adr_names.append("Broadcast")
    adr_states.append(".")
    

    while (zeile.count(',') == 9):
 
        zeile = zeile.split(',')
        l = bytearray()

        for i in range(0,8):
            l.append(int(zeile[i],16))

        adr_64bit.append(bytes(l))
        adr_16bit.append(nw_adr)

        name = zeile[8]
        if len(name) == 0:
            name = "#" + str(cnt + 1)
        adr_names.append(name)

        adr_db.append('      ')
        
        adr_states.append('?')
        
        cnt += 1
        zeile = f.readline()

    #print("importiert wurden: " + str(cnt) + " Adressen")

def set_db(station, db):
    """ Schreibt den Wert der Signalstärke """  
    adr_db[station] = "-{:>02}db ".format(db)


def get_Station_List():
    """ liefert eine formatierte Liste der Stationen zurück """

    stationlist = list()
    for i in range(len(adr_64bit)):
        zeile = ""

        for j in range(8):
            zeile += "{:>02X}".format(adr_64bit[i][j])
            zeile += " "

        zeile += ("--- ")
        for j in range(2):
            zeile += "{:>02X}".format(adr_16bit[i][j])       
            zeile += " "
        zeile += "--- "
        zeile += "{:<20}".format(adr_names[i])

        zeile += adr_db[i]
        zeile += adr_states[i]       
        stationlist.append(zeile)
        
    return stationlist


def get_Station_Names():
    """ liefert eine formatierte Liste mit den Name und dem Status
        der Stationen zurück - zur Verwendung in der Combobox
    """

    stationlist = list()
    for i in range(len(adr_16bit)):
        zeile = ""
        
        for j in range(2):
            zeile += "{:>02X}".format(adr_16bit[i][j])       
            zeile += " "

        zeile += "{:<20}".format(adr_names[i])
#        zeile += adr_states[i]

#        zeile += " - "
        


        stationlist.append(zeile)
        
    return stationlist    

if __name__ == "__main__":

    f = open("xbee.cfg")
    read_adr(f)
    f.close()

##    for i in range(0, len(adr_64bit)):
##        for j in range(0,8):
##            print(adr_64bit[i][j], end = " ")
##        print()
##        
##    for i in range(0, len(adr_16bit)):
##        print(adr_16bit[i])
##
##    for i in range(0, len(adr_names)):
##        print(adr_names[i])
##
##    for i in range(0, len(adr_states)):
##        print(adr_states[i])
        
    stationlist = get_Station_List()
    for i in stationlist:
        print(i)

    stationlist = get_Station_Names()
    for i in stationlist:
        print(i)
    
    while True:
        time.sleep(1)

# --- [ eof ] ------------------------------------------------------------------
